@@ -6,6 +6,8 @@
 
 #pragma once
 
+#include <filesystem>
+
 #include <QAbstractItemModel>
 #include <QObject>
 #include <QPointer>
@@ -41,6 +43,8 @@ class SensorFaceControllerPrivate;
 class SENSORFACES_EXPORT SensorFaceController : public QObject
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_UNCREATABLE("It's not possible to create objects of type SensorFaceController")
     /**
      * A title for the face.
      * @see showTitle
@@ -156,6 +160,10 @@ class SENSORFACES_EXPORT SensorFaceController : public QObject
      * The minimum time that needs to elapse, in milliseconds, between updates of the face.
      */
     Q_PROPERTY(int updateRateLimit READ updateRateLimit WRITE setUpdateRateLimit NOTIFY updateRateLimitChanged)
+    /**
+     * Contains the paths of missing sensors, if there are any.
+     */
+    Q_PROPERTY(QJsonArray missingSensors READ missingSensors NOTIFY missingSensorsChanged)
 
 public:
     /**
@@ -167,7 +175,7 @@ public:
      * @param config The controller uses this config group to read and save the face configuration
      * @param engine This engine will be used for creating the Qml components
      */
-    SensorFaceController(KConfigGroup &config, QQmlEngine *engine);
+    SensorFaceController(KConfigGroup &config, QQmlEngine *engine, QQmlEngine *configEngine);
     ~SensorFaceController() override;
 
     /**
@@ -206,6 +214,8 @@ public:
     QJsonArray lowPrioritySensorIds() const;
     void setLowPrioritySensorIds(const QJsonArray &ids);
 
+    QJsonArray missingSensors() const;
+
     QVariantMap sensorColors() const;
     void setSensorColors(const QVariantMap &colors);
 
@@ -241,6 +251,7 @@ public:
      * Save the current configuration as a preset
      */
     Q_INVOKABLE void savePreset();
+    void savePreset(const std::filesystem::path &path);
     /**
      * Uninstall a specific preset
      */
@@ -268,6 +279,17 @@ public:
      */
     Q_INVOKABLE void reloadFaceConfiguration();
 
+    /**
+     * Replace one sensor with another.
+     *
+     * This replaces a configured sensor with a new one. This replacement happens
+     * inside the configuration, bypassing thing like the sensor properties which
+     * are populated with resolved sensor ids rather than the configured entries.
+     *
+     * You should call @ref reloadConfig once you have made all replacements.
+     */
+    Q_INVOKABLE void replaceSensors(const QString &from, const QString &to);
+
 Q_SIGNALS:
     void faceIdChanged();
     void titleChanged();
@@ -280,6 +302,7 @@ Q_SIGNALS:
     void sensorLabelsChanged();
     void updateRateLimitChanged();
     void faceConfigurationChanged();
+    void missingSensorsChanged();
 
 private:
     const std::unique_ptr<SensorFaceControllerPrivate> d;
